/* 
Modified to start the PSO project!
Copyright Ben Bogart 2003, based on code from Thomas Grill

Still needs the deconstructor, output functions

Segfault in function pso::iterate
Segfault does not occur if one comments out line 314 and line 334
Segfault does occur if either one is left uncommented
*/

// include flext header
#include <flext.h>
#include <stdlib.h>
#include <math.h>

// check for appropriate flext version
#if !defined(FLEXT_VERSION) || (FLEXT_VERSION < 400)
#error You need at least flext version 0.4.0
#endif

// Setup MAX random value
// how to do this without redefining?
#define RAND_MAX 2000000000

class pso:
	// inherit from basic flext class
	public flext_base
{
	// obligatory flext header (class name,base class name)
	FLEXT_HEADER(pso,flext_base)
 
public:
	// constructor 
	pso(int Popsize, int Dimen, int HoodSize);

protected:

	// override default flext help function
        virtual void m_help();

	// flext deconstructor (how to bind this?)
	void clean();

	// PSO Functions
	void init();					// Initialize the PSO 
	void copy_positions();				// Copy the current positions to previous positions
	float minimize(int particle);			// The function to Minimize (sphere)
	void set_pop(int argc,t_atom *argv);  		// Set values to the population
        void set_target(int argc,t_atom *argv);         // Set the PSO target
	void rand_pop(); 				// Randomize the values of the population
	void rand_vel();				// Randomize the velocities
	void iterate(); 				// Iterate the PSO
	void reset();					// Rerun the PSO init
	void output_positions();			// Send out the lists of Particles' positions
  void debug();

	// PSO Init Variables
	int popsize;
	int dimen;
	int hoodsize;
	int gbest;

	// PSO Arrays
        float *current_position;
        float *velocity;
        float *previous_best_position;
        int *neighbors;
        float *previous_best;
        float *target;

private:
	// PSO Callbacks to functions
	
	FLEXT_CALLBACK_V(set_pop);
	FLEXT_CALLBACK_V(set_target);
	FLEXT_CALLBACK(rand_pop); 
	FLEXT_CALLBACK(iterate);
	FLEXT_CALLBACK(reset);
	FLEXT_CALLBACK(m_help);
  FLEXT_CALLBACK(debug);
};

// instantiate the class
FLEXT_NEW_3("pso",pso, int,int,int)


pso::pso(int Popsize, int Dimen, int HoodSize)
{ 
	// var defines
	int i,j,k;

        // Copy Arguments to the class variables
        this->popsize = Popsize;
        this->dimen = Dimen;
        this->hoodsize = HoodSize;

        // Create particle Arrays (single dimentional!)
        this->current_position = new float[this->popsize*this->dimen];
        this->velocity = new float[this->popsize*this->dimen];
        this->previous_best_position = new float[this->popsize*this->dimen];
        this->neighbors = new int[this->popsize*this->hoodsize];
        this->previous_best = new float[this->popsize];

        // Create target array
        this->target = new float[this->dimen];

	// define inlets:
	// first inlet must always by of type anything (or signal for dsp objects)
	AddInAnything();  // add one inlet for any message
	
	// define outlets: Need a list(for each dimen) and bang outlet
	for(i=0; i<dimen;i++)
	{
		AddOutList();
	}
	AddOutBang();
	
	// register methods
	FLEXT_ADDMETHOD_(0,"set_pop",set_pop);
	FLEXT_ADDMETHOD_(0,"rand_pop",rand_pop);
	FLEXT_ADDBANG(0,iterate);
	FLEXT_ADDMETHOD_(0,"reset",reset);
	FLEXT_ADDMETHOD_(0,"help",m_help);
	FLEXT_ADDMETHOD_(0,"target",set_target);
   FLEXT_ADDMETHOD_(0,"debug",debug);

	// init pso
	pso::rand_pop();
	pso::init();
} 

float pso::minimize(int particle)
{
	float result=0;
	int d;

	for (d=0; d< this->dimen; ++d)
	{
	        // Pythagoras - c^2=(a^2+b^2) - for each dimention!! 
		result += pow( this->current_position[particle*this->dimen+d]-(float)this->target[d],2);
	}

	return result;
	post("Minimizing value for particle %d is %f", particle, result);
}

void pso::copy_positions()
{
	int p,d;

	// For each particle and each dimention
        for (p=0; p< this->popsize; ++p)
        {
          for (d=0; d< this->dimen; ++d)
          {
                this->previous_best_position[p*this->dimen+d] = this->current_position[p*this->dimen+d];
          }
        }
}

// Deconstructor
void pso::clean()
{
	// Remove Arrays from memmory space
        delete[] this->current_position;
        delete[] this->velocity;
        delete[] this->previous_best_position;
        delete[] this->neighbors;
        delete[] this->previous_best;
        delete[] this->target;
}

// The population must be set before this function is run!
void pso::init()
{
	int p,n;

	// Do stuff for random # generator
	time_t timer = time(NULL);
	srand((unsigned)timer);

	// set initial population
	pso::rand_vel();
	pso::copy_positions();

	// For each particle
	for (p=0; p< this->popsize; ++p)
	{
		// Who has the best solutions?
		this->previous_best[p]=pso::minimize(p);

		// Create neighborhood topology
		for (n=0; n< this->hoodsize; ++n)
		{
       			/* Connect each neighbor to its previous and next particle */
        		this->neighbors[p*this->hoodsize+n]=p-1+n;

        		/* Make a ring so the first and last particles are connected */
        		if(this->neighbors[p*this->hoodsize+n] < 0)
        		{
          		  this->neighbors[p*this->hoodsize+n]=this->popsize-this->neighbors[p*this->hoodsize+n]-2;
        		} else if(this->neighbors[p*this->hoodsize+n] >= this->popsize)
        		{
          	 	  this->neighbors[p*this->hoodsize+n]=this->neighbors[p*this->hoodsize+n]-this->popsize;
        		} 
		}
	}

	// Set initial global best solution
	this->gbest=0;
}

void pso::set_pop(int argc,t_atom *argv)
{
        int i;

        // If the number of arguments matches the number of particles*dimentions
        if (argc == this->dimen*this->popsize )
        {
                for (i=0; i< this->dimen*this->popsize; i++)
                {
                        this->current_position[i] = GetFloat(argv[i]);
                }
                post("%s: Population Set.", thisName() );
        } else {
                post("%s: There must be %d arguments for each of the %d particles.\n     You entered %d/%d arguments.", thisName(), this->dimen, this->popsize, argc, this->dimen*this->popsize);
        }



	// Do init stuff again
	pso::rand_vel();
	pso::copy_positions();
}

void pso::set_target(int argc,t_atom *argv)
{
	int i;

	// If the number of arguments matches the number of dimentions
	if (argc == this->dimen )
	{
		for (i=0; i<dimen; i++)
		{
			this->target[i] = GetFloat(argv[i]);
		}
		post("%s: Target Set.", thisName() );
	} else {
		post("%s: There must be one argument for each of the %d dimentions.\n     You entered %d arguments.", thisName(), this->dimen, argc);
	}
}

void pso::rand_pop()
{
    int p,d;
    for(p = 0; p < this->popsize; ++p)
    {
        for(d = 0; d < this->dimen; ++d)
	{
	    // set to random numbers 5-10
            this->current_position[p*this->dimen+d] = ((float)rand()/RAND_MAX)*5+5;
	}
    }

    post("%s: Population Randomized.", thisName() ); 
}

void pso::rand_vel()
{ 
    int p,d;
    for(p = 0; p < this->popsize; ++p)
    {
        for(d = 0; d < this->dimen; ++d)
        {
	    // set to random numbers 0-1
            this->velocity[p*this->dimen+d] = ((float)rand()/RAND_MAX);

	    // Set a random number of the velocities to negative
	    if ( ((float)rand()/RAND_MAX) < 0.5 )
		this->velocity[p*this->dimen+d] = this->velocity[p*this->dimen+d]*-1;
        }
    }
}



void pso::iterate()
{
	int p,d,n,g,neighbor;
	float current_minima,rand1,rand2;
	
        // Create constants
        float khi=0.729;
        float HalfPhi=2.05;

	// For each particle
	for (p=0; p< this->popsize; p++)
	{
	  // Who has the best solution?
	  current_minima = pso::minimize(p);

	  // Set initial neighbor with which to compare (first neighbor)
	  g=neighbors[p*this->popsize];

	  // For each neighbor
	  for (n=0; n< this->hoodsize; n++)
	  {
	    neighbor=this->neighbors[p*this->hoodsize+n];
	    if (this->previous_best[neighbor] < this->previous_best[g])
	    {

		// If the current neighbor has a better solution then make g (the best neighbor) the current neighbor
//Line 313
//		g=neighbor; 
	    }
	    if (current_minima < this->previous_best[p])
	    {
		if (current_minima < this->previous_best[this->gbest])
		{
		  this->gbest = p;
		}
		for (d=0; d< this->dimen; d++)
		{
		  this->previous_best_position[p*this->dimen+d] = this->current_position[p*this->dimen+d];
		}
		this->previous_best[p]=current_minima;
	    }

	    for (d=0; d< this->dimen; d++)
	    {
		rand1 = (float)rand()/RAND_MAX;
		rand2 = (float)rand()/RAND_MAX;
//Line 333 
		this->velocity[p*this->dimen+d]=khi*(this->velocity[p*this->dimen+d] + rand1*HalfPhi*(this->previous_best_position[p*this->dimen+d] - this->current_position[p*this->dimen+d]) + rand2*HalfPhi*(this->previous_best_position[g*this->dimen+d] - this->current_position[p*this->dimen+d]));
            	this->current_position[p*this->dimen+d]=this->current_position[p*this->dimen+d]+this->velocity[p*this->dimen+d];
	    }
	  }
	}

	// Have we minimized yet?
	if (this->previous_best[this->gbest] == 0)
	{
		// Also send out a bang
		post("%s: Minimized", thisName());
	}
	post("%f\n", this->previous_best[this->gbest]);
}

void pso::output_positions()
{
}

void pso::reset()
{
	// Reinitialize the PSO
	pso::init();
//	post("%s: Reset.", thisName());
}

void pso::m_help()
{
	post("%s Help:", thisName());
	post("Arguments: [Population Size] [Number of Dimentions] [Neighborhood Size]");
}

void pso::debug()
{
	int p,d;
	for(p = 0; p < this->popsize; ++p)
    	{
        	for(d = 0; d < this->dimen; ++d)
        	{
            		post("%f", this->current_position[p*this->dimen+d]);
        	}
    	}
}
